package com.huiju.estateagents.service.impl;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.huiju.estateagents.base.ResponseBean;
import com.huiju.estateagents.common.CommConstant;
import com.huiju.estateagents.common.StringUtils;
import com.huiju.estateagents.entity.*;
import com.huiju.estateagents.mapper.*;
import com.huiju.estateagents.mapper.TaApartmentImgMapper;
import com.huiju.estateagents.mapper.TaBuildingImgMapper;
import com.huiju.estateagents.mapper.TaHousingResourcesMapper;
import com.huiju.estateagents.mapper.TaPreselectionRecordMapper;
import com.huiju.estateagents.po.TaHousingResourcesPO;
import com.huiju.estateagents.service.ITaHousingResourcesService;
import org.apache.commons.collections.CollectionUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import java.time.LocalDateTime;
import java.util.ArrayList;
import java.util.List;

/**
 * <p>
 * 房源表  服务实现类
 * </p>
 *
 * @author jobob
 * @since 2020-02-10
 */
@Service
public class TaHousingResourcesServiceImpl extends ServiceImpl<TaHousingResourcesMapper, TaHousingResources> implements ITaHousingResourcesService {

    private Logger logger = LoggerFactory.getLogger(TaHousingResourcesServiceImpl.class);

    @Autowired
    private TaHousingResourcesMapper taHousingResourcesMapper;

    @Autowired
    private TaBuildingTermMapper taBuildingTermMapper;

    @Autowired
    private TaBuildingBlockMapper taBuildingBlockMapper;

    @Autowired
    private TaBuildingUnitMapper taBuildingUnitMapper;

    @Autowired
    private TaBuildingFloorMapper taBuildingFloorMapper;

    @Autowired
    private TaBuildingRoomMapper taBuildingRoomMapper;
    
    @Autowired
    private TaBuildingMapper taBuildingMapper;

    @Autowired
    private TaPreselectionRecordMapper taPreselectionRecordMapper;

    @Autowired
    TaBuildingImgMapper taBuildingImgMapper;

    @Autowired
    TaApartmentImgMapper taApartmentImgMapper;


    /**
     * 分页获取房源列表
     *
     * @param pageNum      页码
     * @param pageSize     分页大小
     * @param salesBatchId 销售批次ID
     * @param startPrice
     * @param endPrice
     * @param apartmentId
     * @return
     */
    @Override
    public IPage<TaHousingResourcesPO> listHousingResources(Integer pageNum, Integer pageSize, String salesBatchId, String startPrice, String endPrice, String apartmentId) {
        logger.info("listHousingResources 接收参数：pageNum：{},pageSize：{},salesBatchId：{},startPrice：{},endPrice：{},apartmentId：{}",
                pageNum, pageSize, salesBatchId, startPrice, endPrice, apartmentId);

        IPage<TaHousingResourcesPO> iPage = new Page<>(pageNum, pageSize);
        return taHousingResourcesMapper.listHousingResources(iPage, salesBatchId, startPrice, endPrice, apartmentId);
    }

    /**
     * 获取某销售批次下所有房源户型
     *
     * @param salesBatchId
     * @return
     */
    @Override
    public List<TaBuildingApartment> listBuildApartmentBySalesBatchId(String salesBatchId) {
        logger.info("listBuildApartmentBySalesBatchId 接收参数：salesBatchId：{}", salesBatchId);

        // 获取户型列表
        List<TaBuildingApartment> apartmentList = taHousingResourcesMapper.listBuildApartmentBySalesBatchId(salesBatchId);
        if (CollectionUtils.isEmpty(apartmentList)) {
            return null;
        }

        // 组装户型图片
        List<TaBuildingApartment> resultList = new ArrayList<>();
        List<TaBuildingImg> taBuildingImgList;
        List<TaApartmentImg> buildingImg;
        QueryWrapper<TaApartmentImg> apartmentImgQueryWrapper;
        QueryWrapper<TaBuildingImg> buildingImgQueryWrapper;
        for (TaBuildingApartment apartment : apartmentList) {
            apartmentImgQueryWrapper = new QueryWrapper<>();
            apartmentImgQueryWrapper.eq("apartment_id", apartment.getApartmentId());
            buildingImg = taApartmentImgMapper.selectList(apartmentImgQueryWrapper);
            for (TaApartmentImg apartmentImg : buildingImg) {
                buildingImgQueryWrapper = new QueryWrapper<>();
                buildingImgQueryWrapper.eq("img_id", apartmentImg.getImgId());
                buildingImgQueryWrapper.eq("img_type", CommConstant.BUILDING_IMG_APARMENT);
                taBuildingImgList = taBuildingImgMapper.selectList(buildingImgQueryWrapper);
                apartment.setBuildingImgList(taBuildingImgList);
                resultList.add(apartment);
            }
        }
        return resultList;
    }

    /**
     * 根据房源ID获取房源详情
     *
     * @param personId
     * @param houseId
     * @return
     */
    @Override
    public TaHousingResourcesPO getHousingDetailById(String personId, String houseId) {
        logger.info("getHousingDetailById 接收参数：personId：{}，houseId：{}", personId, houseId);

        // 查询房源详情
        TaHousingResourcesPO resourcesPO = taHousingResourcesMapper.getHousingDetailById(houseId, "1");
        if (resourcesPO == null) {
            return null;
        }

        // 获取该用户是否预选房源信息
        TaPreselectionRecord taPreselectionRecord = taPreselectionRecordMapper.getRecordByPersonIdAndHouseId(personId, houseId);
        if (taPreselectionRecord == null) {
            resourcesPO.setIsPreselect(false);
        } else {
            resourcesPO.setIsPreselect(true);
        }

        // 获取户型图片列表
        List<TaBuildingImg> taBuildingImgList = new ArrayList<>();
        List<TaApartmentImg> buildingImg;
        QueryWrapper<TaBuildingImg> buildingImgQueryWrapper;
        QueryWrapper<TaApartmentImg>  apartmentImgQueryWrapper = new QueryWrapper<>();
        apartmentImgQueryWrapper.eq("apartment_id", resourcesPO.getApartmentId());
        buildingImg = taApartmentImgMapper.selectList(apartmentImgQueryWrapper);
        for (TaApartmentImg apartmentImg : buildingImg) {
            buildingImgQueryWrapper = new QueryWrapper<>();
            buildingImgQueryWrapper.eq("img_id", apartmentImg.getImgId());
            buildingImgQueryWrapper.eq("img_type", CommConstant.BUILDING_IMG_APARMENT);
            taBuildingImgList = taBuildingImgMapper.selectList(buildingImgQueryWrapper);
        }
        if(CollectionUtils.isNotEmpty(taBuildingImgList)){
            resourcesPO.setBuildingImgList(taBuildingImgList);
        }

        return resourcesPO;
    }

    /**
     * 校验楼栋信息存不存在
     *
     * @param taHousingResources
     * @return
     */
    @Override
    public StringBuffer checked(TaHousingResources taHousingResources) {
        StringBuffer result = new StringBuffer();
        //校验期
        String termResult = checckedTerm(taHousingResources);
        result.append(termResult.equals("notExist") ? "期名" + taHousingResources.getTermName() + "不存在,会直接新建。" : termResult );
        //校验栋
        String blockResult = checkedBlock(taHousingResources);
        result.append(blockResult.equals("notExist") ? "楼栋名" + taHousingResources.getBlockName() + "不存在,会直接新建。" : blockResult );
        //校验单元
        String unitResult = checkedUnit(taHousingResources);
        result.append(unitResult.equals("notExist") ? "单元名" + taHousingResources.getUnitName() + "不存在,会直接新建。" : unitResult );
        //校验楼层
        String floorResult = checkedFloor(taHousingResources);
        result.append(floorResult.equals("notExist") ? "楼层名" + taHousingResources.getFloorName() + "不存在,会直接新建。" : floorResult );
        //校验房号
        String roomResult = checkedRoom(taHousingResources);
        result.append(roomResult.equals("notExist") ? "房号" + taHousingResources.getRoomName() + "不存在,会直接新建。" : roomResult );
        return result;
    }

    /**
     * 添加楼栋信息
     *
     * @param taHousingResources
     * @return
     */
    @Override
    public ResponseBean addHouseResources(TaHousingResources taHousingResources) {
        //校验期
        String termResult = checckedTerm(taHousingResources);
        //校验栋
        String blockResult = checkedBlock(taHousingResources);
        //校验单元
        String unitResult = checkedUnit(taHousingResources);
        //校验楼层
        String floorResult = checkedFloor(taHousingResources);
        //校验房号
        String roomResult = checkedRoom(taHousingResources);

        //创建期
        if (!roomResult.equals("notExist")){
            return ResponseBean.error("房间名" + taHousingResources.getRoomName() + "存在。请修改房间名！",ResponseBean.ERROR_UNAVAILABLE);
        }

        //期不存在就添加
        if (termResult.equals("notExist")){
            addTerm(taHousingResources);
        }

        //楼栋不存在就添加
        if (blockResult.equals("notExist")){
            addBlock(taHousingResources);
        }

        //单元不存在就添加
        if (unitResult.equals("notExist")){
            addUnit(taHousingResources);
        }

        //楼层不存在就添加
        if (floorResult.equals("notExist")){
            addFloor(taHousingResources);
        }

        //房间号不存在就添加
        if (roomResult.equals("notExist")){
            addRoom(taHousingResources);
        }
        
        // 获取楼盘名称
        TaBuilding taBuilding = taBuildingMapper.selectById(taHousingResources.getBuildingId());
        taHousingResources.setBuildingName(taBuilding.getBuildingName());
        //添加房源信息
        taHousingResources.setCreateDate(LocalDateTime.now());
        taHousingResourcesMapper.insert(taHousingResources);
        return ResponseBean.success(taHousingResources);
    }

    /**
     * 分页获取房源信息
     *
     * @param salesBatchId
     * @param buildingId
     * @param pageNumber
     * @param pageSize
     * @param orgId
     * @param termName
     * @param blockName
     * @param unitName
     * @param floorName
     * @param roomName
     * @param apartmentId
     * @param status
     * @param houseId
     * @return
     */
    @Override
    public ResponseBean selectPageList(Integer salesBatchId, String buildingId, Integer pageNumber, Integer pageSize, Integer orgId, String termName, String blockName, String unitName, String floorName, String roomName, Integer apartmentId, Integer status, String houseId) {
        IPage<TaHousingResources> pg = new Page<>(pageNumber, pageSize);
        IPage<TaHousingResources> pageData = taHousingResourcesMapper.getHouseResourceList(pg, salesBatchId, buildingId, orgId, termName, blockName, unitName, floorName, roomName, apartmentId, status, houseId);
        List<TaHousingResources> records = pageData.getRecords();
        records.forEach(e -> {
            QueryWrapper<TaPreselectionRecord> queryWrapper = new QueryWrapper<>();
            queryWrapper.eq("status", CommConstant.STATUS_NORMAL);
            queryWrapper.eq("org_id", e.getOrgId());
            queryWrapper.eq("house_id", e.getHouseId());
            List<TaPreselectionRecord> taPreselectionRecords = taPreselectionRecordMapper.selectList(queryWrapper);
            e.setRealHeat(taPreselectionRecords.size());
        });
        return ResponseBean.success(pageData);
    }

    //添加房间号
    private void addRoom(TaHousingResources taHousingResources) {
        TaBuildingRoom taBuildingRoom = new TaBuildingRoom();
        taBuildingRoom.setBuildingId(taHousingResources.getBuildingId());
        taBuildingRoom.setCreateDate(LocalDateTime.now());
        taBuildingRoom.setOrgId(taHousingResources.getOrgId());
        taBuildingRoom.setStatus(CommConstant.STATUS_NORMAL);
        taBuildingRoom.setTermId(taHousingResources.getTermId());
        taBuildingRoom.setBlockId(taHousingResources.getBlockId());
        taBuildingRoom.setUnitId(taHousingResources.getUnitId());
        taBuildingRoom.setFloorId(taHousingResources.getFloorId());
        taBuildingRoom.setRoomName(taHousingResources.getRoomName());
        taBuildingRoomMapper.insert(taBuildingRoom);
        taHousingResources.setRoomId(taBuildingRoom.getRoomId());
    }

    //添加楼层
    private void addFloor(TaHousingResources taHousingResources) {
        TaBuildingFloor taBuildingFloor = new TaBuildingFloor();
        taBuildingFloor.setBuildingId(taHousingResources.getBuildingId());
        taBuildingFloor.setCreateDate(LocalDateTime.now());
        taBuildingFloor.setOrgId(taHousingResources.getOrgId());
        taBuildingFloor.setStatus(CommConstant.STATUS_NORMAL);
        taBuildingFloor.setTermId(taHousingResources.getTermId());
        taBuildingFloor.setBlockId(taHousingResources.getBlockId());
        taBuildingFloor.setUnitId(taHousingResources.getUnitId());
        taBuildingFloor.setFloorName(taHousingResources.getFloorName());
        taBuildingFloorMapper.insert(taBuildingFloor);
        taHousingResources.setFloorId(taBuildingFloor.getFloorId());
    }

    //添加单元
    private void addUnit(TaHousingResources taHousingResources) {
        TaBuildingUnit taBuildingUnit = new TaBuildingUnit();
        taBuildingUnit.setBuildingId(taHousingResources.getBuildingId());
        taBuildingUnit.setCreateDate(LocalDateTime.now());
        taBuildingUnit.setOrgId(taHousingResources.getOrgId());
        taBuildingUnit.setStatus(CommConstant.STATUS_NORMAL);
        taBuildingUnit.setTermId(taHousingResources.getTermId());
        taBuildingUnit.setBlockId(taHousingResources.getBlockId());
        taBuildingUnit.setUnitName(taHousingResources.getUnitName());
        taBuildingUnitMapper.insert(taBuildingUnit);
        taHousingResources.setUnitId(taBuildingUnit.getUnitId());
    }

    //添加楼栋
    private void addBlock(TaHousingResources taHousingResources) {
        TaBuildingBlock taBuildingBlock = new TaBuildingBlock();
        taBuildingBlock.setBuildingId(taHousingResources.getBuildingId());
        taBuildingBlock.setCreateDate(LocalDateTime.now());
        taBuildingBlock.setOrgId(taHousingResources.getOrgId());
        taBuildingBlock.setStatus(CommConstant.STATUS_NORMAL);
        taBuildingBlock.setTermId(taHousingResources.getTermId());
        taBuildingBlock.setBlockName(taHousingResources.getBlockName());
        taBuildingBlockMapper.insert(taBuildingBlock);
        taHousingResources.setBlockId(taBuildingBlock.getBlockId());
    }

    //添加期
    private void addTerm(TaHousingResources taHousingResources) {
        TaBuildingTerm taBuildingTerm = new TaBuildingTerm();
        taBuildingTerm.setBuildingId(taHousingResources.getBuildingId());
        taBuildingTerm.setCreateDate(LocalDateTime.now());
        taBuildingTerm.setOrgId(taHousingResources.getOrgId());
        taBuildingTerm.setStatus(CommConstant.STATUS_NORMAL);
        taBuildingTerm.setTermName(taHousingResources.getTermName());
        taBuildingTermMapper.insert(taBuildingTerm);
        taHousingResources.setTermId(taBuildingTerm.getTermId());
    }

    //校验房号
    private String checkedRoom(TaHousingResources taHousingResources) {
        if (StringUtils.isEmpty(taHousingResources.getRoomName())){
            return "请输入房号。";
        }else{
            QueryWrapper<TaBuildingRoom> queryWrapper = new QueryWrapper<>();
            queryWrapper.eq("org_id",taHousingResources.getOrgId());
            queryWrapper.eq("status",CommConstant.STATUS_NORMAL);
            queryWrapper.eq("building_id",taHousingResources.getBuildingId());
            queryWrapper.eq("term_id",taHousingResources.getTermId());
            queryWrapper.eq("block_id",taHousingResources.getBlockId());
            queryWrapper.eq("unit_id",taHousingResources.getUnitId());
            queryWrapper.eq("floor_id",taHousingResources.getFloorId());
            queryWrapper.eq("room_name",taHousingResources.getRoomName());
            TaBuildingRoom taBuildingRoom = taBuildingRoomMapper.selectOne(queryWrapper);
            if (null != taBuildingRoom){
                taHousingResources.setRoomId(taBuildingRoom.getRoomId());
                return "房间名" + taHousingResources.getRoomName() + "存在。不允许保存，请修改！";
            }else{
                return "notExist";
            }
        }
    }

    //校验楼层
    private String checkedFloor(TaHousingResources taHousingResources) {
        if (StringUtils.isEmpty(taHousingResources.getFloorName())){
            return "请输入楼层名。";
        }else{
            QueryWrapper<TaBuildingFloor> queryWrapper = new QueryWrapper<>();
            queryWrapper.eq("org_id",taHousingResources.getOrgId());
            queryWrapper.eq("status",CommConstant.STATUS_NORMAL);
            queryWrapper.eq("building_id",taHousingResources.getBuildingId());
            queryWrapper.eq("term_id",taHousingResources.getTermId());
            queryWrapper.eq("block_id",taHousingResources.getBlockId());
            queryWrapper.eq("unit_id",taHousingResources.getUnitId());
            queryWrapper.eq("floor_name",taHousingResources.getFloorName());
            TaBuildingFloor taBuildingFloor = taBuildingFloorMapper.selectOne(queryWrapper);
            if (null != taBuildingFloor){
                taHousingResources.setFloorId(taBuildingFloor.getFloorId());
                return "楼层名" + taHousingResources.getFloorName() + "存在。";
            }else{
                return "notExist";
            }
        }
    }

    //校验单元
    private String checkedUnit(TaHousingResources taHousingResources) {
        if (StringUtils.isEmpty(taHousingResources.getUnitName())){
            return "请输入单元名。";
        }else{
            QueryWrapper<TaBuildingUnit> queryWrapper = new QueryWrapper<>();
            queryWrapper.eq("org_id",taHousingResources.getOrgId());
            queryWrapper.eq("status",CommConstant.STATUS_NORMAL);
            queryWrapper.eq("building_id",taHousingResources.getBuildingId());
            queryWrapper.eq("term_id",taHousingResources.getTermId());
            queryWrapper.eq("block_id",taHousingResources.getBlockId());
            queryWrapper.eq("unit_name",taHousingResources.getUnitName());
            TaBuildingUnit taBuildingUnit = taBuildingUnitMapper.selectOne(queryWrapper);
            if (null != taBuildingUnit){
                taHousingResources.setUnitId(taBuildingUnit.getUnitId());
                return "单元名" + taHousingResources.getUnitName() + "存在。";
            }else{
                return "notExist";
            }
        }
    }

    //校验楼盘
    private String checkedBlock(TaHousingResources taHousingResources) {
        if (StringUtils.isEmpty(taHousingResources.getBlockName())){
            return "请输入楼栋名。";
        }else{
            QueryWrapper<TaBuildingBlock> queryWrapper = new QueryWrapper<>();
            queryWrapper.eq("org_id",taHousingResources.getOrgId());
            queryWrapper.eq("status",CommConstant.STATUS_NORMAL);
            queryWrapper.eq("building_id",taHousingResources.getBuildingId());
            queryWrapper.eq("term_id",taHousingResources.getTermId());
            queryWrapper.eq("block_name",taHousingResources.getBlockName());
            TaBuildingBlock taBuildingBlock = taBuildingBlockMapper.selectOne(queryWrapper);
            if (null != taBuildingBlock){
                taHousingResources.setBlockId(taBuildingBlock.getBlockId());
                return "楼栋名" + taHousingResources.getBlockName() + "存在。";
            }else{
                return "notExist";
            }
        }
    }

    //校验期
    private String checckedTerm(TaHousingResources taHousingResources) {
        if (StringUtils.isEmpty(taHousingResources.getTermName())){
            return "请输入期名。";
        }else{
            QueryWrapper<TaBuildingTerm> queryWrapper = new QueryWrapper<>();
            queryWrapper.eq("org_id",taHousingResources.getOrgId());
            queryWrapper.eq("status",CommConstant.STATUS_NORMAL);
            queryWrapper.eq("building_id",taHousingResources.getBuildingId());
            queryWrapper.eq("term_name",taHousingResources.getTermName());
            TaBuildingTerm taBuildingTerm = taBuildingTermMapper.selectOne(queryWrapper);
            if (null != taBuildingTerm){
                taHousingResources.setTermId(taBuildingTerm.getTermId());
                return "期名" + taHousingResources.getTermName() + "存在。";
            }else{
                return "notExist";
            }
        }
    }
}
